@@ -1,37 +1,36 @@
 # Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-import threading
 import argparse
 import os
 import sys
+import threading
 
 from UM.Controller import Controller
 from UM.Message import Message #For typing.
+from UM.PackageManager import PackageManager
 from UM.PluginRegistry import PluginRegistry
-from UM.Mesh.MeshFileHandler import MeshFileHandler
 from UM.Resources import Resources
 from UM.Operations.OperationStack import OperationStack
 from UM.Event import CallFunctionEvent
-from UM.Settings.ContainerRegistry import ContainerRegistry
+import UM.Settings
 import UM.Settings.ContainerStack
 import UM.Settings.InstanceContainer
+from UM.Settings.ContainerRegistry import ContainerRegistry
 from UM.Signal import Signal, signalemitter
 from UM.Logger import Logger
 from UM.Preferences import Preferences
 from UM.View.Renderer import Renderer #For typing.
 from UM.OutputDevice.OutputDeviceManager import OutputDeviceManager
 from UM.i18n import i18nCatalog
-from UM.Workspace.WorkspaceFileHandler import WorkspaceFileHandler
 
-import UM.Settings
-
-from typing import TYPE_CHECKING, Dict, List, Callable, Any, Optional
+from typing import TYPE_CHECKING, List, Callable, Any, Optional
 if TYPE_CHECKING:
     from UM.Backend.Backend import Backend
     from UM.Settings.ContainerStack import ContainerStack
     from UM.Extension import Extension
 
+
 ##  Central object responsible for running the main event loop and creating other central objects.
 #
 #   The Application object is a central object for accessing other important objects. It is also
@@ -41,84 +40,133 @@ if TYPE_CHECKING:
 class Application:
     ##  Init method
     #
-    #   \param name The name of the application.
-    #   \param version Version, formatted as major.minor.rev
-    #   \param build_type Additional version info on the type of build this is,
-    #   such as "master".
+    #   \param name \type{string} The name of the application.
+    #   \param version \type{string} Version, formatted as major.minor.rev
+    #   \param build_type Additional version info on the type of build this is, such as "master".
     #   \param is_debug_mode Whether to run in debug mode.
-    #   \param parser The command line parser to use.
-    def __init__(self, name: str, version: str, build_type: str = "", is_debug_mode: bool = False, parser: argparse.ArgumentParser = None, parsed_command_line: Dict[str, Any] = None, **kwargs) -> None:
-        if Application._instance is not None:
-            raise ValueError("Duplicate singleton creation")
-        if parsed_command_line is None:
-            parsed_command_line = {}
+    def __init__(self, name: str, version: str, build_type: str = "", is_debug_mode: bool = False, **kwargs) -> None:
+        if Application.__instance is not None:
+            raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
+        Application.__instance = self
 
-        # If the constructor is called and there is no instance, set the instance to self.
-        # This is done because we can't make constructor private
-        Application._instance = self
+        super().__init__()  # Call super to make multiple inheritance work.
 
-        self._application_name = name #type: str
+        self._app_name = name #type: str
         self._version = version #type: str
         self._build_type = build_type #type: str
-        if "debug" in parsed_command_line.keys():
-            if not parsed_command_line["debug"] and is_debug_mode:
-                parsed_command_line["debug"] = is_debug_mode
+        self._is_debug_mode = is_debug_mode #type: bool
+        self._is_headless = False #type: bool
+        self._use_external_backend = False #type: bool
 
-        os.putenv("UBUNTU_MENUPROXY", "0")  # For Ubuntu Unity this makes Qt use its own menu bar rather than pass it on to Unity.
+        self._config_lock_filename = "{name}.lock".format(name = self._app_name) # type: str
 
+        self._cli_args = None #type: argparse.Namespace
+        self._cli_parser = argparse.ArgumentParser(prog = self._app_name, add_help = False) #type: argparse.ArgumentParser
+
+        self._main_thread = threading.current_thread() #type: threading.Thread
+
+        self.default_theme = self._app_name  #type: str # Default theme is the application name
+        self._default_language = "en_US" #type: str
+
+        self.change_log_url = "https://github.com/Ultimaker/Uranium" #Where to find a more detailed description of the recent updates.
+
+        self._preferences_filename = None #type: str
+        self._preferences = None #type: Preferences
+
+        self._extensions = [] #type: List[Extension]
+        self._required_plugins = [] #type: List[str]
+
+        self._package_manager_class = PackageManager  # type: type
+        self._package_manager = None  # type: PackageManager
+
+        self._plugin_registry = None #type: PluginRegistry
+        self._container_registry_class = ContainerRegistry #type: type
+        self._container_registry = None #type: ContainerRegistry
+        self._global_container_stack = None #type: ContainerStack
+
+        self._controller = None #type: Controller
+        self._backend = None #type: Backend
+        self._output_device_manager = None #type: OutputDeviceManager
+        self._operation_stack = None #type: OperationStack
+
+        self._visible_messages = [] #type: List[Message]
+        self._message_lock = threading.Lock() #type: threading.Lock
+
+        self._app_install_dir = self.getInstallPrefix() #type: str
+
+    # Adds the command line options that can be parsed by the command line parser.
+    # Can be overridden to add additional command line options to the parser.
+    def addCommandLineOptions(self) -> None:
+        self._cli_parser.add_argument("--version",
+                                      action = "version",
+                                      version = "%(prog)s version: {0}".format(self._version))
+        self._cli_parser.add_argument("--external-backend",
+                                      action = "store_true",
+                                      default = False,
+                                      help = "Use an externally started backend instead of starting it automatically. This is a debug feature to make it possible to run the engine with debug options enabled.")
+        self._cli_parser.add_argument('--headless',
+                                      action = 'store_true',
+                                      default = False,
+                                      help = "Hides all GUI elements.")
+        self._cli_parser.add_argument("--debug",
+                                      action = "store_true",
+                                      default = False,
+                                      help = "Turn on the debug mode by setting this option.")
+
+    def parseCliOptions(self) -> None:
+        self._cli_args = self._cli_parser.parse_args()
+
+        self._is_headless = self._cli_args.headless
+        self._is_debug_mode = self._cli_args.debug or self._is_debug_mode
+        self._use_external_backend = self._cli_args.external_backend
+
+    # Performs initialization that must be done before start.
+    def initialize(self) -> None:
+        # For Ubuntu Unity this makes Qt use its own menu bar rather than pass it on to Unity.
+        os.putenv("UBUNTU_MENUPROXY", "0")
+
+        # Custom signal handling
         Signal._app = self
         Signal._signalQueue = self
-        Resources.ApplicationIdentifier = name
-        Resources.ApplicationVersion = version
+
+        # Initialize Resources. Set the application name and version here because we can only know the actual info
+        # after the __init__() has been called.
+        Resources.ApplicationIdentifier = self._app_name
+        Resources.ApplicationVersion = self._version
 
         Resources.addSearchPath(os.path.join(os.path.dirname(sys.executable), "resources"))
-        Resources.addSearchPath(os.path.join(Application.getInstallPrefix(), "share", "uranium", "resources"))
-        Resources.addSearchPath(os.path.join(Application.getInstallPrefix(), "Resources", "uranium", "resources"))
-        Resources.addSearchPath(os.path.join(Application.getInstallPrefix(), "Resources", self.getApplicationName(), "resources"))
+        Resources.addSearchPath(os.path.join(self._app_install_dir, "share", "uranium", "resources"))
+        Resources.addSearchPath(os.path.join(self._app_install_dir, "Resources", "uranium", "resources"))
+        Resources.addSearchPath(os.path.join(self._app_install_dir, "Resources", self._app_name, "resources"))
 
         if not hasattr(sys, "frozen"):
             Resources.addSearchPath(os.path.join(os.path.abspath(os.path.dirname(__file__)), "..", "resources"))
 
-        self._main_thread = threading.current_thread() #type: threading.Thread
-
-        super().__init__()  # Call super to make multiple inheritance work.
         i18nCatalog.setApplication(self)
 
-        self._renderer = None #type: Renderer
-
         PluginRegistry.addType("backend", self.setBackend)
         PluginRegistry.addType("logger", Logger.addLogger)
         PluginRegistry.addType("extension", self.addExtension)
 
-        self.default_theme = self.getApplicationName() #type: str
+        self._preferences = Preferences()
+        self._preferences.addPreference("general/language", self._default_language)
+        self._preferences.addPreference("general/visible_settings", "")
+        self._preferences.addPreference("general/plugins_to_remove", "")
+        self._preferences.addPreference("general/disabled_plugins", "")
 
-        preferences = Preferences.getInstance()
-        self._preferences = preferences
-        preferences.addPreference("general/language", "en_US")
-        preferences.addPreference("general/visible_settings", "")
-        preferences.addPreference("general/plugins_to_remove", "")
-        preferences.addPreference("general/disabled_plugins", "")
+        self._controller = Controller(self)
+        self._output_device_manager = OutputDeviceManager()
 
-        try:
-            preferences.readFromFile(Resources.getPath(Resources.Preferences, self._application_name + ".cfg"))
-        except FileNotFoundError:
-            pass
+        self._operation_stack = OperationStack(self._controller) # type: OperationStack
 
-        self._controller = Controller(self) #type: Controller
-        self._extensions = [] #type: List[Extension]
-        self._backend = None #type: Backend
-        self._output_device_manager = OutputDeviceManager() #type: OutputDeviceManager
+        self._plugin_registry = PluginRegistry(self)  #type: PluginRegistry
 
-        self._required_plugins = [] #type: List[str]
-
-        self._operation_stack = OperationStack(self.getController()) #type: OperationStack
-
-        self._plugin_registry = PluginRegistry.getInstance() #type: PluginRegistry
-
-        self._plugin_registry.addPluginLocation(os.path.join(Application.getInstallPrefix(), "lib", "uranium"))
+        self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "lib", "uranium"))
+        self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "lib64", "uranium"))
+        self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "lib32", "uranium"))
         self._plugin_registry.addPluginLocation(os.path.join(os.path.dirname(sys.executable), "plugins"))
-        self._plugin_registry.addPluginLocation(os.path.join(Application.getInstallPrefix(), "Resources", "uranium", "plugins"))
-        self._plugin_registry.addPluginLocation(os.path.join(Application.getInstallPrefix(), "Resources", self.getApplicationName(), "plugins"))
+        self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "Resources", "uranium", "plugins"))
+        self._plugin_registry.addPluginLocation(os.path.join(self._app_install_dir, "Resources", self._app_name, "plugins"))
         # Locally installed plugins
         local_path = os.path.join(Resources.getStoragePath(Resources.Resources), "plugins")
         # Ensure the local plugins directory exists
@@ -131,28 +179,40 @@ class Application:
         if not hasattr(sys, "frozen"):
             self._plugin_registry.addPluginLocation(os.path.join(os.path.abspath(os.path.dirname(__file__)), "..", "plugins"))
 
-        self._plugin_registry.setApplication(self)
+        self._container_registry = self._container_registry_class(self)
 
-        ContainerRegistry.setApplication(self)
-        UM.Settings.InstanceContainer.setContainerRegistry(self.getContainerRegistry())
-        UM.Settings.ContainerStack.setContainerRegistry(self.getContainerRegistry())
+        UM.Settings.InstanceContainer.setContainerRegistry(self._container_registry)
+        UM.Settings.ContainerStack.setContainerRegistry(self._container_registry)
 
-        self._command_line_parser = parser #type: argparse.ArgumentParser
-        self._parsed_command_line = parsed_command_line #type: Dict[str, Any]
-        self.parseCommandLine()
+        # Initialize the package manager to remove and install scheduled packages.
+        self._package_manager = self._package_manager_class(self)
 
-        self._visible_messages = [] #type: List[Message]
-        self._message_lock = threading.Lock() #type: threading.Lock
         self.showMessageSignal.connect(self.showMessage)
         self.hideMessageSignal.connect(self.hideMessage)
 
-        self._global_container_stack = None #type: ContainerStack
+    def startSplashWindowPhase(self) -> None:
+        pass
 
-    def getPreferences(self) -> Preferences:
-        return self._preferences
+    def startPostSplashWindowPhase(self) -> None:
+        pass
 
-    def getContainerRegistry(self) -> ContainerRegistry:
-        return ContainerRegistry.getInstance()
+    ##  Run the main event loop.
+    #   This method should be re-implemented by subclasses to start the main event loop.
+    #   \exception NotImplementedError
+    def run(self):
+        self.addCommandLineOptions()
+        self.parseCliOptions()
+        self.initialize()
+
+        self.startSplashWindowPhase()
+        self.startPostSplashWindowPhase()
+
+    def getContainerRegistry(self):
+        return self._container_registry
+
+    ##  Get the lock filename
+    def getApplicationLockFilename(self) -> str:
+        return self._config_lock_filename
 
     ##  Emitted when the application window was closed and we need to shut down the application
     applicationShuttingDown = Signal()
@@ -166,8 +226,9 @@ class Application:
     workspaceLoaded = Signal()
 
     def setGlobalContainerStack(self, stack: "ContainerStack") -> None:
-        self._global_container_stack = stack
-        self.globalContainerStackChanged.emit()
+        if self._global_container_stack != stack:
+            self._global_container_stack = stack
+            self.globalContainerStackChanged.emit()
 
     def getGlobalContainerStack(self) -> Optional["ContainerStack"]:
         return self._global_container_stack
@@ -182,26 +243,26 @@ class Application:
         raise NotImplementedError
 
     ##  Get the version of the application
-    #   \returns version \type{string}
     def getVersion(self) -> str:
         return self._version
 
-    @classmethod
-    def getStaticVersion(cls) -> str:
-        return "unknown"
-
-    ##  Get the buildtype of the application
-    #   \returns version \type{string}
+    ##  Get the build type of the application
     def getBuildType(self) -> str:
         return self._build_type
 
     def getIsDebugMode(self) -> bool:
-        return self.getCommandLineOption("debug")
+        return self._is_debug_mode
+
+    def getIsHeadLess(self) -> bool:
+        return self._is_headless
+
+    def getUseExternalBackend(self) -> bool:
+        return self._use_external_backend
 
     visibleMessageAdded = Signal()
 
     ##  Hide message by ID (as provided by built-in id function)
-    def hideMessageById(self, message_id: int):
+    def hideMessageById(self, message_id: int) -> None:
         # If a user and the application tries to close same message dialog simultaneously, message_id could become an empty
         # string, and then the application will raise an error when trying to do "int(message_id)".
         # So we check the message_id here.
@@ -227,40 +288,35 @@ class Application:
     def _loadPlugins(self) -> None:
         pass
 
-    def getCommandLineOption(self, name: str, default: str = None) -> Any:
-        if name not in self._parsed_command_line.keys():
-            self.parseCommandLine()
-            Logger.log("d", "Command line options: %s", str(self._parsed_command_line))
-
-        return self._parsed_command_line.get(name, default)
-
     ##  Get name of the application.
-    #   \returns application_name \type{string}
+    #   \returns app_name \type{string}
     def getApplicationName(self) -> str:
-        return self._application_name
+        return self._app_name
 
     ##  Get the preferences.
     #   \return preferences \type{Preferences}
     def getPreferences(self) -> Preferences:
-        return Preferences.getInstance()
+        return self._preferences
+
+    def savePreferences(self) -> None:
+        if self._preferences_filename:
+            self._preferences.writeToFile(self._preferences_filename)
+        else:
+            Logger.log("i", "Preferences filename not set. Unable to save file.")
 
     ##  Get the currently used IETF language tag.
     #   The returned tag is during runtime used to translate strings.
     #   \returns Language tag.
     def getApplicationLanguage(self) -> str:
-        override_lang = os.getenv("URANIUM_LANGUAGE")
-        if override_lang:
-            return override_lang
+        language = os.getenv("URANIUM_LANGUAGE")
+        if not language:
+            language = self._preferences.getValue("general/language")
+        if not language:
+            language = os.getenv("LANGUAGE")
+        if not language:
+            language = self._default_language
 
-        preflang = Preferences.getInstance().getValue("general/language")
-        if preflang:
-            return preflang
-
-        env_lang = os.getenv("LANGUAGE")
-        if env_lang:
-            return env_lang
-
-        return "en_US"
+        return language
 
     ##  Application has a list of plugins that it *must* have. If it does not have these, it cannot function.
     #   These plugins can not be disabled in any way.
@@ -297,16 +353,6 @@ class Application:
     def getOutputDeviceManager(self) -> OutputDeviceManager:
         return self._output_device_manager
 
-    ##  Includes eg. last checks before entering the main event loop.
-    def preRun(self) -> None:
-        return None
-
-    ##  Run the main event loop.
-    #   This method should be re-implemented by subclasses to start the main event loop.
-    #   \exception NotImplementedError
-    def run(self) -> None:
-        raise NotImplementedError("Run must be implemented by application")
-
     ##  Return an application-specific Renderer object.
     #   \exception NotImplementedError
     def getRenderer(self) -> Renderer:
@@ -333,56 +379,6 @@ class Application:
     def getMainThread(self) -> threading.Thread:
         return self._main_thread
 
-    ##  Return the singleton instance of the application object
-    @classmethod
-    def getInstance(cls, **kwargs) -> "Application":
-        # Note: Explicit use of class name to prevent issues with inheritance.
-        if not Application._instance:
-            Application._instance = cls(**kwargs)
-
-        return Application._instance
-
-    def getCommandlineParser(self, with_help: bool = False) -> argparse.ArgumentParser:
-        if not self._command_line_parser:
-            self._command_line_parser = argparse.ArgumentParser(prog = self.getApplicationName(), add_help = with_help) #pylint: disable=bad-whitespace
-            self.addCommandLineOptions(self._command_line_parser, parsed_command_line = self._parsed_command_line)
-        return self._command_line_parser
-
-    def parseCommandLine(self) -> None:
-        parser = self.getCommandlineParser()
-        new_parsed_args = vars(parser.parse_known_args()[0])
-        new_parsed_args.update(self._parsed_command_line)
-        self._parsed_command_line = new_parsed_args
-
-    ##  Can be overridden to add additional command line options to the parser.
-    #
-    #   \param parser The parser that will parse the command line.
-    @classmethod
-    def addCommandLineOptions(cls, parser: argparse.ArgumentParser, parsed_command_line: Dict[str, str] = None) -> None:
-        if parsed_command_line is None:
-            parsed_command_line = {}
-
-        parser.add_argument("--version",
-                            action = "version",
-                            version = "%(prog)s {0}".format(cls.getStaticVersion()))
-
-        parser.add_argument("--external-backend",
-                            dest = "external-backend",
-                            action = "store_true",
-                            default = False,
-                            help = "Use an externally started backend instead of starting it automatically. This is a debug feature to make it possible to run the engine with debug options enabled.")
-
-        parser.add_argument('--headless',
-                            action = 'store_true',
-                            default = False,
-                            help = "Hides all GUI elements.")
-
-        if "debug" not in parsed_command_line.keys():
-            parser.add_argument("--debug",
-                                action = "store_true",
-                                default = False,
-                                help = "Turn on the debug mode by setting this option.")
-
     def addExtension(self, extension: "Extension") -> None:
         self._extensions.append(extension)
 
@@ -396,4 +392,8 @@ class Application:
         else:
             return os.path.abspath(os.path.join(os.path.dirname(sys.executable), ".."))
 
-    _instance = None    # type: Application
+    __instance = None   # type: Application
+
+    @classmethod
+    def getInstance(cls, *args, **kwargs) -> "Application":
+        return cls.__instance
