@@ -2,6 +2,7 @@
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import imp
+import json
 import os
 import shutil  # For deleting plugin directories;
 import stat    # For setting file permissions correctly;
@@ -15,13 +16,14 @@ import types
 
 from PyQt5.QtCore import QObject, pyqtSlot, QUrl, pyqtProperty, pyqtSignal
 
-from UM.Resources import Resources
-from UM.PluginObject import PluginObject  # For type hinting
+from UM.i18n import i18nCatalog
+from UM.Logger import Logger
 from UM.Platform import Platform
+from UM.PluginError import PluginNotFoundError, InvalidMetaDataError
+from UM.PluginObject import PluginObject  # For type hinting
+from UM.Resources import Resources
 from UM.Version import Version
 
-from UM.i18n import i18nCatalog
-import json
 i18n_catalog = i18nCatalog("uranium")
 
 if TYPE_CHECKING:
@@ -39,10 +41,15 @@ if TYPE_CHECKING:
 #   [plugins]: docs/plugins.md
 
 class PluginRegistry(QObject):
-    APIVersion = 4
+    APIVersion = 5
+
+    def __init__(self, application: "Application", parent: QObject = None) -> None:
+        if PluginRegistry.__instance is not None:
+            raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
+        PluginRegistry.__instance = self
 
-    def __init__(self, parent: QObject = None) -> None:
         super().__init__(parent)
+        self._application = application #type: Application
 
         self._all_plugins = []        # type: List[str]
         self._metadata = {}           # type: Dict[str, Dict[str, Any]]
@@ -66,17 +73,14 @@ class PluginRegistry(QObject):
         self._plugin_locations = []  # type: List[str]
         self._folder_cache = {}      # type: Dict[str, List[Tuple[str, str]]]
 
-        self._application = None #type: Application
-        self._supported_file_types = {"umplugin": "Uranium Plugin"}
-
-        # File to store plugin info, such as which ones to install/remove and which ones are disabled.
-        # Cannot load this here because we don't know the actual Application name yet, so it will result in incorrect
-        # directory name if we try to get it from Resources now.
-        self._plugin_config_filename = None #type: Optional[str]
+        self._supported_file_types = {"umplugin": "Uranium Plugin"} # type: Dict[str, str]
 
     def initializeBeforePluginsAreLoaded(self) -> None:
         config_path = Resources.getConfigStoragePath()
-        self._plugin_config_filename = os.path.join(os.path.abspath(config_path), "plugins.json")
+
+        # File to store plugin info, such as which ones to install/remove and which ones are disabled.
+        # At this point we can load this here because we already know the actual Application name, so the directory name
+        self._plugin_config_filename = os.path.join(os.path.abspath(config_path), "plugins.json") # type: str
 
         from UM.Settings.ContainerRegistry import ContainerRegistry
         container_registry = ContainerRegistry.getInstance()
@@ -95,9 +99,10 @@ class PluginRegistry(QObject):
             Logger.logException("e", "Failed to load plugin configuration file '%s'", self._plugin_config_filename)
 
         # Also load data from preferences, where the plugin info used to be saved
-        preferences = Preferences.getInstance()
+        preferences = self._application.getPreferences()
         disabled_plugins = preferences.getValue("general/disabled_plugins")
         disabled_plugins = disabled_plugins.split(",") if disabled_plugins else []
+        disabled_plugins = [plugin for plugin in disabled_plugins if len(plugin.strip()) > 0]
         for plugin_id in disabled_plugins:
             if plugin_id not in self._disabled_plugins:
                 self._disabled_plugins.append(plugin_id)
@@ -123,7 +128,7 @@ class PluginRegistry(QObject):
         self._savePluginData()
 
     def initializeAfterPluginsAreLoaded(self) -> None:
-        preferences = Preferences.getInstance()
+        preferences = self._application.getPreferences()
 
         # Remove the old preferences settings from preferences
         preferences.resetPreference("general/disabled_plugins")
@@ -132,13 +137,18 @@ class PluginRegistry(QObject):
     def _savePluginData(self) -> None:
         from UM.Settings.ContainerRegistry import ContainerRegistry
         container_registry = ContainerRegistry.getInstance()
-        with container_registry.lockFile():
-            with open(self._plugin_config_filename, "w", encoding = "utf-8") as f:
-                data = json.dumps({"disabled": self._disabled_plugins,
-                                   "to_install": self._plugins_to_install,
-                                   "to_remove": self._plugins_to_remove,
-                                   })
-                f.write(data)
+        try:
+            with container_registry.lockFile():
+                with open(self._plugin_config_filename, "w", encoding = "utf-8") as f:
+                    data = json.dumps({"disabled": self._disabled_plugins,
+                                       "to_install": self._plugins_to_install,
+                                       "to_remove": self._plugins_to_remove,
+                                       })
+                    f.write(data)
+        except:
+            # Since we're writing to file (and waiting for a lock), there are a few things that can go wrong.
+            # There is no need to crash the application for this, but it is a failure that we want to log.
+            Logger.log("e", "Unable to save the plugin data.")
 
     # TODO:
     # - [ ] Improve how metadata is stored. It should not be in the 'plugin' prop
@@ -161,7 +171,7 @@ class PluginRegistry(QObject):
     #   registry. Cura uses this method to add 3rd-party plugins.
     def addExternalPlugins(self, plugin_list: List[Dict[str, Any]]) -> None:
         for plugin in plugin_list:
-            # Add the plugin id to the the all plugins list if not already there:
+            # Add the plugin id to the all plugins list if not already there:
             if plugin["id"] not in self._all_plugins:
                 self._all_plugins.append(plugin["id"])
 
@@ -256,13 +266,6 @@ class PluginRegistry(QObject):
                 plugins.append(plugin_id)
         return plugins
 
-    #   Get the singleton instance of this class:
-    @classmethod
-    def getInstance(cls) -> "PluginRegistry":
-        if not cls._instance:
-            cls._instance = PluginRegistry()
-        return cls._instance
-
     #   Get the metadata for a certain plugin:
     #   NOTE: InvalidMetaDataError is raised when no metadata can be found or
     #         the metadata misses the right keys.
@@ -276,10 +279,6 @@ class PluginRegistry(QObject):
 
         return self._metadata[plugin_id]
 
-    #   Get the list of plugin locations:
-    def getPluginLocations(self) -> List[str]:
-        return self._plugin_locations
-
     @pyqtSlot(str, result="QVariantMap")
     def installPlugin(self, plugin_path: str) -> Optional[Dict[str, str]]:
         plugin_path = QUrl(plugin_path).toLocalFile()
@@ -360,16 +359,15 @@ class PluginRegistry(QObject):
             # Get the plugin metadata:
             plugin_metadata = self.getMetaData(plugin_id)
 
-            # Add the plugin to the list:
-            self._all_plugins.append(plugin_id)
-            self._plugins_installed.append(plugin_id)
-
             # Save all metadata to the metadata dictionary:
             self._metadata[plugin_id] = plugin_metadata
             if metadata is None or self._subsetInDict(self._metadata[plugin_id], metadata):
                 #
                 try:
                     self.loadPlugin(plugin_id)
+                    # Add the plugin to the list after actually load the plugin:
+                    self._all_plugins.append(plugin_id)
+                    self._plugins_installed.append(plugin_id)
                 except PluginNotFoundError:
                     pass
 
@@ -405,13 +403,6 @@ class PluginRegistry(QObject):
             del self._metadata[plugin_id]
             return
 
-        #HACK: For OctoPrint plug-in version 3.2.2, it broke the start-up sequence when auto-connecting.
-        #Remove this hack once we've increased the API version number to something higher than 4.
-        version = self._metadata[plugin_id].get("plugin", {}).get("version", "0.0.0")
-        if plugin_id == "OctoPrintPlugin" and Version(version) < Version("3.3.0"):
-            Logger.log("e", "Plugin OctoPrintPlugin version {version} was disabled because it was using an old API for network connection.".format(version = version))
-            return
-
         try:
             to_register = plugin.register(self._application) #type: ignore #We catch AttributeError on this in case register() doesn't exist.
             if not to_register:
@@ -436,10 +427,6 @@ class PluginRegistry(QObject):
         except Exception as e:
             Logger.logException("e", "Error loading plugin %s:", plugin_id)
 
-    #   Set the central application object:
-    def setApplication(self, app: "Application") -> None:
-        self._application = app
-
     #   Uninstall a plugin with a given ID:
     @pyqtSlot(str, result="QVariantMap")
     def uninstallPlugin(self, plugin_id: str) -> Dict[str, str]:
@@ -540,7 +527,7 @@ class PluginRegistry(QObject):
     ##  Try to find a module implementing a plugin
     #   \param plugin_id The name of the plugin to find
     #   \returns module if it was found None otherwise
-    def _findPlugin(self, plugin_id: str) -> types.ModuleType:
+    def _findPlugin(self, plugin_id: str) -> Optional[types.ModuleType]:
         location = None
         for folder in self._plugin_locations:
             location = self._locatePlugin(plugin_id, folder)
@@ -584,9 +571,9 @@ class PluginRegistry(QObject):
             if file == plugin_id and os.path.exists(os.path.join(file_path, "__init__.py")):
                 return folder
             else:
-                file_path = self._locatePlugin(plugin_id, file_path)
-                if file_path:
-                    return file_path
+                plugin_path = self._locatePlugin(plugin_id, file_path)
+                if plugin_path:
+                    return plugin_path
 
         return None
 
@@ -670,13 +657,6 @@ class PluginRegistry(QObject):
                 return False
         return True
 
-    #===============================================================================
-    # GRAVEYARD
-    # Methods in the graveyard are no longer used and can eventually be removed and
-    # forgotten by the ages. They aren't yet though because their memories still
-    # live on in the hearts of other classes.
-    #===============================================================================
-
     ##  Get a speficic plugin object given an ID. If not loaded, load it.
     #   \param plugin_id \type{string} The ID of the plugin object to get.
     def getPluginObject(self, plugin_id: str) -> PluginObject:
@@ -686,7 +666,6 @@ class PluginRegistry(QObject):
             raise PluginNotFoundError(plugin_id)
         return self._plugin_objects[plugin_id]
 
-    # Plugin object stuff is definitely considered depreciated.
     def _addPluginObject(self, plugin_object: PluginObject, plugin_id: str, plugin_type: str) -> None:
         plugin_object.setPluginId(plugin_id)
         self._plugin_objects[plugin_id] = plugin_object
@@ -721,20 +700,13 @@ class PluginRegistry(QObject):
         file_types.append(i18n_catalog.i18nc("@item:inlistbox", "All Files (*)"))
         return file_types
 
-    @pyqtSlot(str, result = bool)
-    def isPluginFile(self, plugin_path: str) -> bool:
-        extension = os.path.splitext(plugin_path)[1].strip(".")
-        if extension.lower() in self._supported_file_types.keys():
-            return True
-        return False
-
     ##  Get the path to a plugin.
     #
     #   \param plugin_id \type{string} The ID of the plugin.
     #   \return \type{string} The absolute path to the plugin or an empty string if the plugin could not be found.
     def getPluginPath(self, plugin_id: str) -> Optional[str]:
         if plugin_id in self._plugins:
-            plugin = self._plugins[plugin_id]
+            plugin = self._plugins.get(plugin_id)
         else:
             plugin = self._findPlugin(plugin_id)
 
@@ -773,6 +745,9 @@ class PluginRegistry(QObject):
         if plugin_type in cls._type_register_map:
             del cls._type_register_map[plugin_type]
 
-
     _type_register_map = {}  # type: Dict[str, Callable[[Any], None]]
-    _instance = None    # type: PluginRegistry
+    __instance = None    # type: PluginRegistry
+
+    @classmethod
+    def getInstance(cls, *args, **kwargs) -> "PluginRegistry":
+        return cls.__instance
