@@ -1,8 +1,9 @@
 # Copyright (c) 2015 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 
 from PyQt5.QtQml import qmlRegisterType, qmlRegisterSingletonType, qmlRegisterUncreatableType
 
+from UM.Qt.Bindings import StageModel
 from UM.Qt.Duration import Duration, DurationFormat
 
 from . import MainWindow
@@ -116,3 +117,10 @@ class Bindings:
         # Additions after 2.3;
         qmlRegisterSingletonType(WorkspaceFileHandler, "UM", 1, 3, "WorkspaceFileHandler", WorkspaceFileHandler.getInstance)
         qmlRegisterSingletonType(OpenGLContextProxy.OpenGLContextProxy, "UM", 1, 3, "OpenGLContextProxy", Bindings.createOpenGLContextProxy)
+
+        # Additions after 3.1
+        qmlRegisterType(StageModel.StageModel, "UM", 1, 4, "StageModel")
+
+    @staticmethod
+    def addRegisterType(class_type: type, qml_import_name: str, major_version: int, minor_version: int, class_name: str) -> None:
+        qmlRegisterType(class_type, qml_import_name, major_version, minor_version, class_name)
