@@ -17,6 +17,7 @@ from UM.Scene.Iterator.BreadthFirstIterator import BreadthFirstIterator
 from UM.Scene.SceneNode import SceneNode
 from UM.Signal import Signal, signalemitter
 from UM.i18n import i18nCatalog
+from UM.Platform import Platform
 
 i18n_catalog = i18nCatalog("uranium")
 
@@ -109,7 +110,7 @@ class Scene:
 
     def getAllCameras(self) -> List[Camera]:
         cameras = []
-        for node in BreadthFirstIterator(self._root): #type: ignore
+        for node in BreadthFirstIterator(self._root):  # type: ignore
             if isinstance(node, Camera):
                 cameras.append(node)
         return cameras
@@ -123,7 +124,7 @@ class Scene:
         else:
             Logger.log("w", "Couldn't find camera with name [%s] to activate!" % name)
 
-    ##  Signal. Emitted whenever something in the scene changes.
+    ##  Signal that is emitted whenever something in the scene changes.
     #   \param object The object that triggered the change.
     sceneChanged = Signal()
 
@@ -133,13 +134,13 @@ class Scene:
     #
     #   \return The object if found, or None if not.
     def findObject(self, object_id: int) -> Optional["SceneNode"]:
-        for node in BreadthFirstIterator(self._root): #type: ignore
+        for node in BreadthFirstIterator(self._root):  # type: ignore
             if id(node) == object_id:
                 return node
         return None
 
     def findCamera(self, name: str) -> Optional[Camera]:
-        for node in BreadthFirstIterator(self._root): #type: ignore
+        for node in BreadthFirstIterator(self._root):  # type: ignore
             if isinstance(node, Camera) and node.getName() == name:
                 return node
         return None
@@ -147,28 +148,33 @@ class Scene:
     ##  Add a file to be watched for changes.
     #   \param file_path The path to the file that must be watched.
     def addWatchedFile(self, file_path: str) -> None:
-        self._file_watcher.addPath(file_path)
+        # The QT 5.10.0 issue, only on Windows. Cura crashes after loading a stl file from USB/sd-card/Cloud-based drive
+        if not Platform.isWindows():
+            self._file_watcher.addPath(file_path)
 
     ##  Remove a file so that it will no longer be watched for changes.
     #   \param file_path The path to the file that must no longer be watched.
     def removeWatchedFile(self, file_path: str) -> None:
-        self._file_watcher.removePath(file_path)
+        # The QT 5.10.0 issue, only on Windows. Cura crashes after loading a stl file from USB/sd-card/Cloud-based drive
+        if not Platform.isWindows():
+            self._file_watcher.removePath(file_path)
 
     ##  Triggered whenever a file is changed that we currently have loaded.
     def _onFileChanged(self, file_path: str) -> None:
-        if not os.path.isfile(file_path): #File doesn't exist any more.
+        if not os.path.isfile(file_path) or os.path.getsize(file_path) == 0:  # File doesn't exist any more, or it is empty
             return
 
-        #Multiple nodes may be loaded from the same file at different stages. Reload them all.
-        from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator #To find which nodes to reload when files have changed.
-        modified_nodes = (node for node in DepthFirstIterator(self.getRoot()) if node.getMeshData() and node.getMeshData().getFileName() == file_path) #type: ignore
+        # Multiple nodes may be loaded from the same file at different stages. Reload them all.
+        from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator  # To find which nodes to reload when files have changed.
+        modified_nodes = [node for node in DepthFirstIterator(self.getRoot()) if node.getMeshData() and node.getMeshData().getFileName() == file_path]  # type: ignore
 
         if modified_nodes:
-            message = Message(i18n_catalog.i18nc("@info", "Would you like to reload {filename}?").format(filename = os.path.basename(file_path)),
+            self._reload_message = Message(i18n_catalog.i18nc("@info", "Would you like to reload {filename}?").format(filename = os.path.basename(file_path)),
                               title = i18n_catalog.i18nc("@info:title", "File has been modified"))
-            message.addAction("reload", i18n_catalog.i18nc("@action:button", "Reload"), icon = None, description = i18n_catalog.i18nc("@action:description", "This will trigger the modified files to reload from disk."))
-            message.actionTriggered.connect(functools.partialmethod(self._reloadNodes, modified_nodes))
-            message.show()
+            self._reload_message.addAction("reload", i18n_catalog.i18nc("@action:button", "Reload"), icon = "", description = i18n_catalog.i18nc("@action:description", "This will trigger the modified files to reload from disk."))
+            self._reload_callback = functools.partial(self._reloadNodes, modified_nodes)
+            self._reload_message.actionTriggered.connect(self._reload_callback)
+            self._reload_message.show()
 
     ##  Reloads a list of nodes after the user pressed the "Reload" button.
     #   \param nodes The list of nodes that needs to be reloaded.
@@ -177,12 +183,17 @@ class Scene:
     def _reloadNodes(self, nodes: List["SceneNode"], message: str, action: str) -> None:
         if action != "reload":
             return
+        self._reload_message.hide()
         for node in nodes:
-            if not os.path.isfile(node.getMeshData().getFileName()): #File doesn't exist any more.
-                continue
-            job = ReadMeshJob(node.getMeshData().getFileName())
-            job.finished.connect(functools.partialmethod(self._reloadJobFinished, node))
-            job.start()
+            meshdata = node.getMeshData()
+            if meshdata:
+                filename = meshdata.getFileName()
+                if not filename or not os.path.isfile(filename):  # File doesn't exist any more.
+                    continue
+                job = ReadMeshJob(filename)
+                self._reload_finished_callback = functools.partial(self._reloadJobFinished, node)
+                job.finished.connect(self._reload_finished_callback)
+                job.start()
 
     ##  Triggered when reloading has finished.
     #
