@@ -1,5 +1,5 @@
 # Copyright (c) 2015 Ultimaker B.V.
-# Uranium is released under the terms of the AGPLv3 or higher.
+# Uranium is released under the terms of the LGPLv3 or higher.
 from typing import List
 
 from UM.Signal import Signal
@@ -46,7 +46,7 @@ class Selection:
     def getBoundingBox(cls):
         bounding_box = None  # don't start with an empty bounding box, because that includes (0,0,0)
         for node in cls.__selection:
-            if type(node) is not SceneNode:
+            if not isinstance(node, SceneNode):
                 continue
 
             if not bounding_box:
@@ -130,13 +130,7 @@ class Selection:
 
     @classmethod
     def _onTransformationChanged(cls, node):
-        cls.__selection_center = Vector.Null
-
-        for object in cls.__selection:
-            cls.__selection_center = cls.__selection_center + object.getWorldPosition()
-
-        cls.__selection_center = cls.__selection_center / len(cls.__selection)
-
+        cls.__selection_center = cls.getBoundingBox().center
         cls.selectionCenterChanged.emit()
 
     __selection = []    # type: List[SceneNode]
