@@ -7,34 +7,33 @@ import os
 import pickle #For serializing/deserializing Python classes to binary files
 import re #For finding containers with asterisks in the constraints and for detecting backup files.
 import time
-from typing import Any, cast, Dict, List, Optional, Set, TYPE_CHECKING
+from typing import Any, cast, Dict, List, Optional, Set, Type, TYPE_CHECKING
 
 import UM.Dictionary
 import UM.FlameProfiler
 from UM.LockFile import LockFile
 from UM.Logger import Logger
 from UM.MimeTypeDatabase import MimeType, MimeTypeDatabase
-from UM.PluginObject import PluginObject #For typing.
 from UM.PluginRegistry import PluginRegistry #To register the container type plug-ins and container provider plug-ins.
 from UM.Resources import Resources
 from UM.Settings.ContainerFormatError import ContainerFormatError
 from UM.Settings.ContainerProvider import ContainerProvider
+from UM.Settings.constant_instance_containers import empty_container
 from . import ContainerQuery
 from UM.Settings.ContainerStack import ContainerStack
 from UM.Settings.DefinitionContainer import DefinitionContainer
 from UM.Settings.InstanceContainer import InstanceContainer
 from UM.Settings.Interfaces import ContainerInterface, ContainerRegistryInterface, DefinitionContainerInterface
-from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext #For typing.
 from UM.Signal import Signal, signalemitter
 
 if TYPE_CHECKING:
-    from UM.Application import Application
-
-CONFIG_LOCK_FILENAME = "uranium.lock"
+    from UM.PluginObject import PluginObject
+    from UM.Qt.QtApplication import QtApplication
 
 # The maximum amount of query results we should cache
 MaxQueryCacheSize = 1000
 
+
 ##  Central class to manage all setting providers.
 #
 #   This class aggregates all data from all container providers. If only the
@@ -43,25 +42,31 @@ MaxQueryCacheSize = 1000
 #   appropriate providers.
 @signalemitter
 class ContainerRegistry(ContainerRegistryInterface):
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
+    def __init__(self, application: "QtApplication") -> None:
+        if ContainerRegistry.__instance is not None:
+            raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
+        ContainerRegistry.__instance = self
 
-        self._emptyInstanceContainer = _EmptyInstanceContainer("empty")
+        super().__init__()
 
-        #Sorted list of container providers (keep it sorted by sorting each time you add one!).
-        self._providers = [] # type: List[ContainerProvider]
+        self._application = application  # type: QtApplication
+
+        self._emptyInstanceContainer = empty_container  # type: InstanceContainer
+
+        # Sorted list of container providers (keep it sorted by sorting each time you add one!).
+        self._providers = []  # type: List[ContainerProvider]
         PluginRegistry.addType("container_provider", self.addProvider)
 
-        self.metadata = {} # type: Dict[str, Dict[str, Any]]
-        self._containers = {} # type: Dict[str, ContainerInterface]
+        self.metadata = {}  # type: Dict[str, Dict[str, Any]]
+        self._containers = {}  # type: Dict[str, ContainerInterface]
         self._wrong_container_ids = set() # type: Set[str]  # Set of already known wrong containers that must be skipped
-        self.source_provider = {} # type: Dict[str, Optional[ContainerProvider]] #Where each container comes from.
+        self.source_provider = {}  # type: Dict[str, Optional[ContainerProvider]]  # Where each container comes from.
         # Ensure that the empty container is added to the ID cache.
         self.metadata["empty"] = self._emptyInstanceContainer.getMetaData()
         self._containers["empty"] = self._emptyInstanceContainer
         self.source_provider["empty"] = None
         self._resource_types = {"definition": Resources.DefinitionContainers}  # type: Dict[str, int]
-        self._query_cache = collections.OrderedDict() #type: collections.OrderedDict # This should really be an ordered set but that does not exist...
+        self._query_cache = collections.OrderedDict()  # type: collections.OrderedDict # This should really be an ordered set but that does not exist...
 
         #Since queries are based on metadata, we need to make sure to clear the cache when a container's metadata changes.
         self.containerMetaDataChanged.connect(self._clearQueryCache)
@@ -91,7 +96,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     ##  Adds a container provider to search through containers in.
     def addProvider(self, provider: ContainerProvider) -> None:
         self._providers.append(provider)
-        #Re-sort every time. It's quadratic, but there shouldn't be that many providers anyway...
+        # Re-sort every time. It's quadratic, but there shouldn't be that many providers anyway...
         self._providers.sort(key = lambda provider: PluginRegistry.getInstance().getMetaData(provider.getPluginId())["container_provider"].get("priority", 0))
 
     ##  Find all DefinitionContainer objects matching certain criteria.
@@ -111,7 +116,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   \return A list of metadata dictionaries matching the search criteria, or
     #   an empty list if nothing was found.
     def findDefinitionContainersMetadata(self, **kwargs: Any) -> List[Dict[str, Any]]:
-        return cast(List[Dict[str, Any]], self.findContainersMetadata(container_type = DefinitionContainer, **kwargs))
+        return self.findContainersMetadata(container_type = DefinitionContainer, **kwargs)
 
     ##  Find all InstanceContainer objects matching certain criteria.
     #
@@ -130,7 +135,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   \return A list of metadata dictionaries matching the search criteria, or
     #   an empty list if nothing was found.
     def findInstanceContainersMetadata(self, **kwargs: Any) -> List[Dict[str, Any]]:
-        return cast(List[Dict[str, Any]], self.findContainersMetadata(container_type = InstanceContainer, **kwargs))
+        return self.findContainersMetadata(container_type = InstanceContainer, **kwargs)
 
     ##  Find all ContainerStack objects matching certain criteria.
     #
@@ -148,7 +153,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   \return A list of metadata dictionaries matching the search criteria, or
     #   an empty list if nothing was found.
     def findContainerStacksMetadata(self, **kwargs: Any) -> List[Dict[str, Any]]:
-        return cast(List[Dict[str, Any]], self.findContainersMetadata(container_type = ContainerStack, **kwargs))
+        return self.findContainersMetadata(container_type = ContainerStack, **kwargs)
 
     ##  Find all container objects matching certain criteria.
     #
@@ -162,13 +167,13 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   list if nothing was found.
     @UM.FlameProfiler.profile
     def findContainers(self, *, ignore_case: bool = False, **kwargs: Any) -> List[ContainerInterface]:
-        #Find the metadata of the containers and grab the actual containers from there.
+         # Find the metadata of the containers and grab the actual containers from there.
         results_metadata = self.findContainersMetadata(ignore_case = ignore_case, **kwargs)
         result = []
         for metadata in results_metadata:
-            if metadata["id"] in self._containers: #Already loaded, so just return that.
+            if metadata["id"] in self._containers:  # Already loaded, so just return that.
                 result.append(self._containers[metadata["id"]])
-            else: # Metadata is loaded, but not the actual data.
+            else:  # Metadata is loaded, but not the actual data.
                 if metadata["id"] in self._wrong_container_ids:
                     Logger.logException("e", "Error when loading container {container_id}: This is a weird container, probably some file is missing".format(container_id = metadata["id"]))
                     continue
@@ -199,12 +204,12 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   \return A list of metadata dictionaries matching the search criteria, or
     #   an empty list if nothing was found.
     def findContainersMetadata(self, *, ignore_case: bool = False, **kwargs: Any) -> List[Dict[str, Any]]:
-        #Create the query object.
+        # Create the query object.
         query = ContainerQuery.ContainerQuery(self, ignore_case = ignore_case, **kwargs)
         candidates = None
 
         if "id" in kwargs and kwargs["id"] is not None and "*" not in kwargs["id"] and not ignore_case:
-            if kwargs["id"] not in self.metadata: #If we're looking for an unknown ID, try to lazy-load that one.
+            if kwargs["id"] not in self.metadata:  # If we're looking for an unknown ID, try to lazy-load that one.
                 if kwargs["id"] not in self.source_provider:
                     for candidate in self._providers:
                         if kwargs["id"] in candidate.getAllIds():
@@ -222,13 +227,13 @@ class ContainerRegistry(ContainerRegistryInterface):
                 self.metadata[metadata["id"]] = metadata
                 self.source_provider[metadata["id"]] = provider
 
-            #Since IDs are the primary key and unique we can now simply request the candidate and check if it matches all requirements.
+            # Since IDs are the primary key and unique we can now simply request the candidate and check if it matches all requirements.
             if kwargs["id"] not in self.metadata:
-                return [] #No result, so return an empty list.
+                return []  # No result, so return an empty list.
             candidates = [self.metadata[kwargs["id"]]]
 
         if query.isHashable() and query in self._query_cache:
-            #If the exact same query is in the cache, we can re-use the query result.
+            # If the exact same query is in the cache, we can re-use the query result.
             self._query_cache.move_to_end(query) #Query was used, so make sure to update its position so that it doesn't get pushed off as a rarely-used query.
             return self._query_cache[query].getResult()
 
@@ -245,7 +250,7 @@ class ContainerRegistry(ContainerRegistryInterface):
                 # to be discarded.
                 self._query_cache.popitem(last = False)
 
-        return query.getResult()
+        return cast(List[Dict[str, Any]], query.getResult())  # As the execute of the query is done, result won't be none.
 
     ##  Specialized find function to find only the modified container objects
     #   that also match certain criteria.
@@ -264,20 +269,20 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   \return A list of containers matching the search criteria, or an empty
     #   list if nothing was found.
     def findDirtyContainers(self, *, ignore_case: bool = False, **kwargs: Any) -> List[ContainerInterface]:
-        #Find the metadata of the containers and grab the actual containers from there.
+        # Find the metadata of the containers and grab the actual containers from there.
         #
-        #We could apply the "is in self._containers" filter and the "isDirty" filter
-        #to this metadata find function as well to filter earlier, but since the
-        #filters in findContainersMetadata are applied in arbitrary order anyway
-        #this will have very little effect except to prevent a list copy.
+        # We could apply the "is in self._containers" filter and the "isDirty" filter
+        # to this metadata find function as well to filter earlier, but since the
+        # filters in findContainersMetadata are applied in arbitrary order anyway
+        # this will have very little effect except to prevent a list copy.
         results_metadata = self.findContainersMetadata(ignore_case = ignore_case, **kwargs)
 
         result = []
         for metadata in results_metadata:
-            if metadata["id"] not in self._containers: #Not yet loaded, so it can't be dirty.
+            if metadata["id"] not in self._containers:  # Not yet loaded, so it can't be dirty.
                 continue
             candidate = self._containers[metadata["id"]]
-            if hasattr(candidate, "isDirty") and candidate.isDirty(): #type: ignore #Check for hasattr because only InstanceContainers and Stacks have this method.
+            if hasattr(candidate, "isDirty") and candidate.isDirty():  # type: ignore #Check for hasattr because only InstanceContainers and Stacks have this method.
                 result.append(self._containers[metadata["id"]])
         return result
 
@@ -294,7 +299,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     def isReadOnly(self, container_id: str) -> bool:
         provider = self.source_provider.get(container_id)
         if not provider:
-            return False #If no provider had the container, that means that the container was only in memory. Then it's always modifiable.
+            return False  # If no provider had the container, that means that the container was only in memory. Then it's always modifiable.
         return provider.isReadOnly(container_id)
 
     ##  Returns whether a container is completely loaded or not.
@@ -308,11 +313,10 @@ class ContainerRegistry(ContainerRegistryInterface):
     ##  Load the metadata of all available definition containers, instance
     #   containers and container stacks.
     def loadAllMetadata(self) -> None:
-        for provider in self._providers: #Automatically sorted by the priority queue.
-            for container_id in list(provider.getAllIds()): #Make copy of all IDs since it might change during iteration.
+        for provider in self._providers:  # Automatically sorted by the priority queue.
+            for container_id in list(provider.getAllIds()):  # Make copy of all IDs since it might change during iteration.
                 if container_id not in self.metadata:
-                    from UM.Qt.QtApplication import QtApplication
-                    QtApplication.getInstance().processEvents() #Update the user interface because loading takes a while. Specifically the loading screen.
+                    self._application.processEvents()  # Update the user interface because loading takes a while. Specifically the loading screen.
                     metadata = provider.loadMetadata(container_id)
                     if metadata is None:
                         continue
@@ -327,17 +331,16 @@ class ContainerRegistry(ContainerRegistryInterface):
     #   that were already added when the first call to this method happened will not be re-added.
     @UM.FlameProfiler.profile
     def load(self) -> None:
-        #Disable garbage collection to speed up the loading (at the cost of memory usage).
+        # Disable garbage collection to speed up the loading (at the cost of memory usage).
         gc.disable()
         resource_start_time = time.time()
 
-        with self.lockCache(): #Because we might be writing cache files.
+        with self.lockCache():  # Because we might be writing cache files.
             for provider in self._providers:
-                for container_id in list(provider.getAllIds()): #Make copy of all IDs since it might change during iteration.
+                for container_id in list(provider.getAllIds()):  # Make copy of all IDs since it might change during iteration.
                     if container_id not in self._containers:
-                        #Update UI while loading.
-                        from UM.Qt.QtApplication import QtApplication
-                        QtApplication.getInstance().processEvents() #Update the user interface because loading takes a while. Specifically the loading screen.
+                        # Update UI while loading.
+                        self._application.processEvents()  # Update the user interface because loading takes a while. Specifically the loading screen.
                         try:
                             self._containers[container_id] = provider.loadContainer(container_id)
                         except:
@@ -365,9 +368,15 @@ class ContainerRegistry(ContainerRegistryInterface):
         if container_id not in self.source_provider:
             self.source_provider[container_id] = None #Added during runtime.
         self._clearQueryCacheByContainer(container)
-        self.containerAdded.emit(container)
         Logger.log("d", "Container [%s] added.", container_id)
 
+        # containerAdded is a custom signal and can trigger direct calls to its subscribers. This should be avoided
+        # because with the direct calls, the subscribers need to know everything about what it tries to do to avoid
+        # triggering this signal again, which eventually can end up exceeding the max recursion limit.
+        # We avoid the direct calls here to make sure that the subscribers do not need to take into account any max
+        # recursion problem.
+        self._application.callLater(self.containerAdded.emit, container)
+
     @UM.FlameProfiler.profile
     def removeContainer(self, container_id: str) -> None:
         # Here we only need to check metadata because a container may not be loaded but its metadata must have been
@@ -466,7 +475,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     #
     #   \param container An instance of the container type to add.
     @classmethod
-    def addContainerType(cls, container: PluginObject) -> None:
+    def addContainerType(cls, container: "PluginObject") -> None:
         plugin_id = container.getPluginId()
         metadata = PluginRegistry.getInstance().getMetaData(plugin_id)
         if "settings_container" not in metadata or "mimetype" not in metadata["settings_container"]:
@@ -530,6 +539,8 @@ class ContainerRegistry(ContainerRegistryInterface):
         with self.lockFile():
             # Save base files first
             for instance in self.findDirtyContainers(container_type = InstanceContainer):
+                if instance.getMetaDataEntry("removed"):
+                    continue
                 if instance.getId() == instance.getMetaData().get("base_file"):
                     self.saveContainer(instance)
 
@@ -542,7 +553,7 @@ class ContainerRegistry(ContainerRegistryInterface):
     # Load a binary cached version of a DefinitionContainer
     def _loadCachedDefinition(self, definition_id: str, path: str) -> None:
         try:
-            cache_path = Resources.getPath(Resources.Cache, "definitions", self.getApplication().getVersion(), definition_id)
+            cache_path = Resources.getPath(Resources.Cache, "definitions", self._application.getVersion(), definition_id)
 
             cache_mtime = os.path.getmtime(cache_path)
             definition_mtime = os.path.getmtime(path)
@@ -570,7 +581,7 @@ class ContainerRegistry(ContainerRegistryInterface):
 
     # Store a cached version of a DefinitionContainer
     def _saveCachedDefinition(self, definition: DefinitionContainer):
-        cache_path = Resources.getStoragePath(Resources.Cache, "definitions", self.getApplication().getVersion(), definition.id)
+        cache_path = Resources.getStoragePath(Resources.Cache, "definitions", self._application.getVersion(), definition.id)
 
         # Ensure the cache path exists
         os.makedirs(os.path.dirname(cache_path), exist_ok=True)
@@ -623,11 +634,11 @@ class ContainerRegistry(ContainerRegistryInterface):
     ##  Get the lock filename including full path
     #   Dependent on when you call this function, Resources.getConfigStoragePath may return different paths
     def getLockFilename(self) -> str:
-        return Resources.getStoragePath(Resources.Resources, CONFIG_LOCK_FILENAME)
+        return Resources.getStoragePath(Resources.Resources, self._application.getApplicationLockFilename())
 
     ##  Get the cache lock filename including full path.
     def getCacheLockFilename(self) -> str:
-        return Resources.getStoragePath(Resources.Cache, CONFIG_LOCK_FILENAME)
+        return Resources.getStoragePath(Resources.Cache, self._application.getApplicationLockFilename())
 
     ##  Contextmanager to create a lock file and remove it afterwards.
     def lockFile(self) -> LockFile:
@@ -646,25 +657,6 @@ class ContainerRegistry(ContainerRegistryInterface):
             wait_msg = "Waiting for lock file in cache directory to disappear."
         )
 
-    ##  Get the singleton instance for this class.
-    @classmethod
-    def getInstance(cls) -> "ContainerRegistry":
-        # Note: Explicit use of class name to prevent issues with inheritance.
-        if not ContainerRegistry.__instance:
-            ContainerRegistry.__instance = cls()
-        return ContainerRegistry.__instance
-
-    @classmethod
-    def setApplication(cls, application: "Application") -> None:
-        cls.__application = application
-
-    @classmethod
-    def getApplication(cls) -> "Application":
-        return cls.__application
-
-    __application = None    # type: Application
-    __instance = None  # type: ContainerRegistry
-
     __container_types = {
         "definition": DefinitionContainer,
         "instance": InstanceContainer,
@@ -676,24 +668,13 @@ class ContainerRegistry(ContainerRegistryInterface):
         "application/x-uranium-instancecontainer": InstanceContainer,
         "application/x-uranium-containerstack": ContainerStack,
         "application/x-uranium-extruderstack": ContainerStack
-    }
+    }  # type: Dict[str, Type[ContainerInterface]]
+
+    __instance = None  # type: ContainerRegistry
+
+    @classmethod
+    def getInstance(cls, *args, **kwargs) -> "ContainerRegistry":
+        return cls.__instance
+
 
 PluginRegistry.addType("settings_container", ContainerRegistry.addContainerType)
-
-
-class _EmptyInstanceContainer(InstanceContainer):
-    def isDirty(self) -> bool:
-        return False
-
-    def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None) -> Any:
-        return None
-
-    def setProperty(self, key: str, property_name: str, property_value: Any, container: ContainerInterface = None, set_from_cache: bool = False) -> None:
-        Logger.log("e", "Setting property %s of container %s which should remain empty", key, self.getName())
-        return
-
-    def getConfigurationType(self) -> str:
-        return ""  # FIXME: not sure if this is correct
-
-    def serialize(self, ignored_metadata_keys: Optional[set] = None) -> str:
-        return "[general]\n version = " + str(InstanceContainer.Version) + "\n name = empty\n definition = fdmprinter\n"
