@@ -4,7 +4,7 @@
 import configparser
 import io
 import copy
-from typing import Any, cast, Dict, List, Optional, Tuple
+from typing import Any, cast, Dict, List, Optional, Set, Tuple
 
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal
 from PyQt5.QtQml import QQmlEngine #To take ownership of this class ourselves.
@@ -19,15 +19,19 @@ from UM.MimeTypeDatabase import MimeTypeDatabase, MimeType
 from UM.Settings.Interfaces import ContainerInterface, ContainerRegistryInterface
 from UM.Settings.SettingInstance import SettingInstance
 
+
 class InvalidInstanceError(Exception):
     pass
 
+
 class IncorrectInstanceVersionError(Exception):
     pass
 
+
 class DefinitionNotFoundError(Exception):
     pass
 
+
 MimeTypeDatabase.addMimeType(
     MimeType(
         name = "application/x-uranium-instancecontainer",
@@ -206,7 +210,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             "id": self.getId(),
             "name": self.getName(),
             "definition": self.getMetaData().get("definition"),
-            "version": self.getMetaData().get("version", 0)
+            "version": self.getMetaData().get("version", 0),
+            "container_type": InstanceContainer
         }
         self._metadata.update(metadata)
         self._dirty = True
@@ -221,20 +226,6 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     def getMetaDataEntry(self, entry: str, default = None) -> Any:
         return self._metadata.get(entry, default)
 
-    ##  Add a new entry to the metadata of this container.
-    #
-    #   \param key \type{str} The key of the new entry.
-    #   \param value The value of the new entry.
-    #
-    #   \note This does nothing if the key already exists.
-    def addMetaDataEntry(self, key: str, value: Any) -> None:
-        if key not in self._metadata:
-            self._metadata[key] = value
-            self._dirty = True
-            self.metaDataChanged.emit(self)
-        else:
-            Logger.log("w", "Meta data with key %s was already added.", key)
-
     ##  Set a metadata entry to a certain value.
     #
     #   \param key The key of the metadata entry to set.
@@ -242,12 +233,10 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     #
     #   \note This does nothing if the key is not already added to the metadata.
     def setMetaDataEntry(self, key: str, value: Any) -> None:
-        if key in self._metadata:
+        if key not in self._metadata or self._metadata[key] != value:
             self._metadata[key] = value
             self._dirty = True
             self.metaDataChanged.emit(self)
-        else:
-            Logger.log("w", "Meta data with key %s was not found. Unable to change.", key)
 
     ##  Check if this container is dirty, that is, if it changed from deserialization.
     def isDirty(self) -> bool:
@@ -345,7 +334,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             instance.propertyChanged.connect(self.propertyChanged)
             self._instances[instance.definition.key] = instance
 
-        self._instances[key].setProperty(property_name, property_value, container)
+        # Do not emit any signal if the value is set from cache
+        self._instances[key].setProperty(property_name, property_value, container, emit_signals = not set_from_cache)
 
         if not set_from_cache:
             self.setDirty(True)
@@ -362,12 +352,12 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
     ##  Get all the keys of the instances of this container
     #   \returns list of keys
-    def getAllKeys(self) -> List[str]:
+    def getAllKeys(self) -> Set[str]:
         keys = set(key for key in self._instances)
         if self._cached_values:
             # If we only want the keys and the actual values are still cached, just get the keys from the cache.
             keys.update(self._cached_values.keys())
-        return list(keys)
+        return keys
 
     ##  Create a new InstanceContainer with the same contents as this container
     #
@@ -442,7 +432,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     def _readAndValidateSerialized(cls, serialized: str) -> configparser.ConfigParser:
         # Disable comments in the ini files, so text values can start with a ;
         # without being removed as a comment
-        parser = configparser.ConfigParser(interpolation=None, comment_prefixes = ())
+        parser = configparser.ConfigParser(interpolation = None, comment_prefixes = ())
         parser.read_string(serialized)
 
         has_general = "general" in parser
@@ -499,7 +489,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
         try:
             parser_version = int(parser["general"]["version"])
-        except ValueError: #Version number is not integer.
+        except ValueError:  # Version number is not integer.
             raise IncorrectInstanceVersionError("Reported version {0} is not an integer.".format(parser["general"]["version"]))
         if parser_version != self.Version:
             raise IncorrectInstanceVersionError("Reported version {0} but expected version {1}".format(str(parser_version), self.Version))
@@ -538,7 +528,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     #   instead.
     @classmethod
     def deserializeMetadata(cls, serialized: str, container_id: str) -> List[Dict[str, Any]]:
-        serialized = cls._updateSerialized(serialized) #Update to most recent version.
+        serialized = cls._updateSerialized(serialized)  # Update to most recent version.
         parser = configparser.ConfigParser(interpolation = None)
         parser.read_string(serialized)
 
@@ -675,7 +665,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     def __str__(self) -> str:
         return "<InstanceContainer '{container_id}' ('{name}')>".format(container_id = self.getId(), name = self.getName())
 
-    ##  Send postponed emits
+    ##  Send the postponed emits
     #   These emits are collected from the option postpone_emit.
     #   Note: the option can be implemented for all functions modifying the container.
     def sendPostponedEmits(self) -> None:
@@ -684,7 +674,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             signal.emit(*signal_arg)
 
 
-_containerRegistry = ContainerRegistryInterface()   # type:  ContainerRegistryInterface
+_containerRegistry = ContainerRegistryInterface()  # type:  ContainerRegistryInterface
+
 
 def setContainerRegistry(registry: ContainerRegistryInterface) -> None:
     global _containerRegistry
