@@ -2,21 +2,19 @@
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import collections
-import itertools
 import os.path
+from typing import List
 
 from PyQt5.QtCore import Qt, QAbstractListModel, QVariant, QModelIndex, QObject, pyqtProperty, pyqtSignal
 from UM.FlameProfiler import pyqtSlot
 
 from UM.Logger import Logger
-from UM.Preferences import Preferences
-from UM.Resources import Resources
 from UM.Settings import SettingRelation
 from UM.i18n import i18nCatalog
 from UM.Application import Application
 
 from UM.Settings.ContainerRegistry import ContainerRegistry
-from UM.Settings.SettingDefinition import SettingDefinition, DefinitionPropertyType
+from UM.Settings.SettingDefinition import SettingDefinition
 
 ##  Model that provides a flattened list of the tree of SettingDefinition objects in a DefinitionContainer
 #
@@ -49,10 +47,13 @@ class SettingDefinitionsModel(QAbstractListModel):
         self._visible = set()
         self._exclude = set()
 
-        self._show_all = False
-        self._show_ancestors = False
+        self._show_all = False  # type: bool
+        self._show_ancestors = False  # type: bool
         self._visibility_handler = None
 
+        self._update_visible_row_scheduled = False  # type: bool
+        self._destroyed = False  # type: bool
+
         self._filter_dict = {}
 
         self._role_names = {
@@ -66,14 +67,23 @@ class SettingDefinitionsModel(QAbstractListModel):
             self._role_names[index] = name.encode()
             index += 1
 
+        self.destroyed.connect(self._onDestroyed)
+
     ##  Emitted whenever the showAncestors property changes.
     showAncestorsChanged = pyqtSignal()
 
-    def setShowAncestors(self, show_ancestors):
+    def _onDestroyed(self) -> None:
+        self._destroyed = True
+
+    @pyqtSlot(bool)
+    def setDestroyed(self, value):
+        self._destroyed = value
+
+    def setShowAncestors(self, show_ancestors: bool) -> None:
         if show_ancestors != self._show_ancestors:
             self._show_ancestors = show_ancestors
             self._update()
-            self.showAncestorsChanged.emit()
+            self._scheduleUpdateVisibleRows()
 
     @pyqtProperty(bool, fset=setShowAncestors, notify=showAncestorsChanged)
     # Should we still show ancestors, even if filter says otherwise?
@@ -125,11 +135,11 @@ class SettingDefinitionsModel(QAbstractListModel):
         return self._root_key
 
     ##  Set the showAll property.
-    def setShowAll(self, show):
+    def setShowAll(self, show: bool) -> None:
         if show != self._show_all:
             self._show_all = show
             self.showAllChanged.emit()
-            self._updateVisibleRows()
+            self._scheduleUpdateVisibleRows()
 
     ##  Emitted when the showAll property changes.
     showAllChanged = pyqtSignal()
@@ -170,7 +180,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         if exclude != self._exclude:
             self._exclude = exclude
             self.excludeChanged.emit()
-            self._updateVisibleRows()
+            self._scheduleUpdateVisibleRows()
 
     ##  Emitted whenever the exclude property changes
     excludeChanged = pyqtSignal()
@@ -181,7 +191,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         return list(self._exclude)
 
     ##  Set the expanded property
-    def setExpanded(self, expanded):
+    def setExpanded(self, expanded: List[str]) -> None:
         new_expanded = set()
         for item in expanded:
             if item == "*":
@@ -194,7 +204,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         if new_expanded != self._expanded:
             self._expanded = new_expanded
             self.expandedChanged.emit()
-            self._updateVisibleRows()
+            self._scheduleUpdateVisibleRows()
 
     ##  Emitted whenever the exclude property changes
     expandedChanged = pyqtSignal()
@@ -231,7 +241,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         if filter_dict != self._filter_dict:
             self._filter_dict = filter_dict
             self.filterChanged.emit()
-            self._updateVisibleRows()
+            self._scheduleUpdateVisibleRows()
 
     filterChanged = pyqtSignal()
 
@@ -245,7 +255,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         if key not in self._expanded:
             self._expanded.add(key)
             self.expandedChanged.emit()
-            self._updateVisibleRows()
+            self._scheduleUpdateVisibleRows()
 
     ##  Show the children of a specified SettingDefinition and all children of those settings as well.
     @pyqtSlot(str)
@@ -282,7 +292,7 @@ class SettingDefinitionsModel(QAbstractListModel):
                 self.collapse(child.key)
 
         self.expandedChanged.emit()
-        self._updateVisibleRows()
+        self._scheduleUpdateVisibleRows()
 
     ##  Show a single SettingDefinition.
     @pyqtSlot(str)
@@ -360,6 +370,11 @@ class SettingDefinitionsModel(QAbstractListModel):
 
         index = self._definition_list.index(definitions[0])
 
+        # Make sure self._row_index_list is populated
+        if self._update_visible_row_scheduled:
+            self._update_visible_row_scheduled = False
+            self._updateVisibleRows()
+
         try:
             return self._row_index_list.index(index)
         except ValueError:
@@ -484,6 +499,12 @@ class SettingDefinitionsModel(QAbstractListModel):
 
         self._updateVisibleRows()
 
+    # Schedules to call _updateVisibleRows() later.
+    def _scheduleUpdateVisibleRows(self):
+        if not self._update_visible_row_scheduled:
+            self._update_visible_row_scheduled = True
+            Application.getInstance().callLater(self._updateVisibleRows)
+
     ##  Force updating the model.
     @pyqtSlot()
     def forceUpdate(self):
@@ -513,7 +534,7 @@ class SettingDefinitionsModel(QAbstractListModel):
         else:
             self._definition_list = self._container.findDefinitions()
 
-        self._updateVisibleRows()
+        self._scheduleUpdateVisibleRows()
 
         self.endResetModel()
 
@@ -522,6 +543,15 @@ class SettingDefinitionsModel(QAbstractListModel):
     # This will compute the difference between the old state and the new state and
     # insert/remove rows as appropriate.
     def _updateVisibleRows(self):
+        # This function is scheduled on the Qt event loop. By the time this is called, this object can already been
+        # destroyed because the owner QML widget was destroyed or so. We cannot cancel a call that has been scheduled,
+        # so in this case, we should do nothing if this object has already been destroyed.
+        if self._destroyed:
+            return
+
+        # Reset the scheduled flag
+        self._update_visible_row_scheduled = False
+
         currently_visible = set(self._row_index_list) # A set of currently visible items
 
         new_visible = set() # A new set of visible items
@@ -614,7 +644,7 @@ class SettingDefinitionsModel(QAbstractListModel):
                 continue
 
             if child.key in self._visible:
-                if Application.getInstance().getGlobalContainerStack().getProperty(child.key, "enabled"):
+                if self._container.getProperty(child.key, "enabled"):
                     return True
 
             if self._isAnyDescendantVisible(child):
