@@ -1,7 +1,7 @@
 # Copyright (c) 2017 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal
+from PyQt5.QtCore import QObject, QTimer, pyqtProperty, pyqtSignal
 from PyQt5.QtQml import QQmlPropertyMap
 from UM.FlameProfiler import pyqtSlot
 
@@ -29,7 +29,6 @@ class SettingPropertyProvider(QObject):
 
         self._property_map = QQmlPropertyMap(self)
 
-        self._stack_id = ""
         self._stack = None
         self._key = ""
         self._relations = set()
@@ -40,43 +39,58 @@ class SettingPropertyProvider(QObject):
         self._remove_unused_value = True
         self._validator = None
 
-        self.storeIndexChanged.connect(self._update)
+        self._update_timer = QTimer()
+        self._update_timer.setInterval(100)
+        self._update_timer.setSingleShot(True)
+        self._update_timer.timeout.connect(self._update)
 
-    ##  Set the containerStackId property.
-    def setContainerStackId(self, stack_id):
-        if stack_id == self._stack_id:
-            return # No change.
-
-        self._stack_id = stack_id
+        self.storeIndexChanged.connect(self._storeIndexChanged)
 
+    def setContainerStack(self, stack):
         if self._stack:
             self._stack.propertiesChanged.disconnect(self._onPropertiesChanged)
-            self._stack.containersChanged.disconnect(self._update)
+            self._stack.containersChanged.disconnect(self._containersChanged)
 
-        if self._stack_id:
-            if self._stack_id == "global":
-                self._stack = Application.getInstance().getGlobalContainerStack()
-            else:
-                stacks = ContainerRegistry.getInstance().findContainerStacks(id = self._stack_id)
-                if stacks:
-                    self._stack = stacks[0]
+        self._stack = stack
 
-            if self._stack:
-                self._stack.propertiesChanged.connect(self._onPropertiesChanged)
-                self._stack.containersChanged.connect(self._update)
-        else:
-            self._stack = None
+        if self._stack:
+            self._stack.propertiesChanged.connect(self._onPropertiesChanged)
+            self._stack.containersChanged.connect(self._containersChanged)
 
         self._validator = None
         self._update()
-        self.containerStackIdChanged.emit()
+        self.containerStackChanged.emit()
+
+    ##  Set the containerStackId property.
+    def setContainerStackId(self, stack_id):
+        if stack_id == self.containerStackId:
+            return  # No change.
+
+        if stack_id:
+            if stack_id == "global":
+                self.setContainerStack(Application.getInstance().getGlobalContainerStack())
+            else:
+                stacks = ContainerRegistry.getInstance().findContainerStacks(id = stack_id)
+                if stacks:
+                    self.setContainerStack(stacks[0])
+        else:
+            self.setContainerStack(None)
 
     ##  Emitted when the containerStackId property changes.
     containerStackIdChanged = pyqtSignal()
     ##  The ID of the container stack we should query for property values.
     @pyqtProperty(str, fset = setContainerStackId, notify = containerStackIdChanged)
     def containerStackId(self):
-        return self._stack_id
+        try:
+            return self._stack.id
+        except:
+            return ""
+
+    containerStackChanged = pyqtSignal()
+
+    @pyqtProperty(QObject, fset=setContainerStack, notify=containerStackChanged)
+    def containerStack(self):
+        return self._stack
 
     removeUnusedValueChanged = pyqtSignal()
 
@@ -171,6 +185,12 @@ class SettingPropertyProvider(QObject):
                     old_value = self.getPropertyValue(property_name, index)
 
                     key_state = str(self._stack.getContainer(self._store_index).getProperty(self._key, "state"))
+
+                    # The old_value might be a SettingFunction, like round(), sum(), etc.
+                    #  In this case retrieve the value to compare
+                    if isinstance(old_value, SettingFunction):
+                        old_value = old_value(self._stack)
+
                     # sometimes: old value is int, property_value is float
                     # (and the container is not removed, so the revert button appears)
                     if str(old_value) == str(property_value) and key_state != "InstanceState.Calculated":
@@ -261,10 +281,12 @@ class SettingPropertyProvider(QObject):
 
             if self._stack.getProperty(key, "state") != InstanceState.User:
                 value_used_count += 1
+                break
 
             # If the setting has a formula the value is still used.
             if isinstance(self._stack.getRawProperty(key, "value"), SettingFunction):
                 value_used_count += 1
+                break
 
         self._value_used = relation_count == 0 or (relation_count > 0 and value_used_count != 0)
         return self._value_used
@@ -307,6 +329,15 @@ class SettingPropertyProvider(QObject):
         self._value_used = None
         self.isValueUsedChanged.emit()
 
+    def _updateDelayed(self, container = None):
+        self._update_timer.start()
+
+    def _containersChanged(self, container = None):
+        self._updateDelayed(container = container)
+
+    def _storeIndexChanged(self, container = None):
+        self._updateDelayed(container = container)
+
     ##  Updates the self._stack_levels field, which indicates at which levels in
     #   the stack the property is set.
     def _updateStackLevels(self):
