@@ -77,14 +77,14 @@ class SettingDefinition:
     #   \param container \type{DefinitionContainerInterface} The container of this setting. Defaults to None.
     #   \param parent \type{SettingDefinition} The parent of this setting. Defaults to None.
     #   \param i18n_catalog \type{i18nCatalog} The translation catalog to use for this setting. Defaults to None.
-    def __init__(self, key: str, container: Optional[DefinitionContainerInterface] = None, parent: Optional["SettingDefinition"] = None, i18n_catalog: i18nCatalog = None) -> None:
+    def __init__(self, key: str, container: Optional[DefinitionContainerInterface] = None, parent: Optional["SettingDefinition"] = None, i18n_catalog: Optional[i18nCatalog] = None) -> None:
         super().__init__()
 
         self._key = key  # type: str
         self._container = container # type: Optional[DefinitionContainerInterface]
         self._parent = parent   # type:  Optional["SettingDefinition"]
 
-        self._i18n_catalog = i18n_catalog  # type: i18nCatalog
+        self._i18n_catalog = i18n_catalog  # type: Optional[i18nCatalog]
 
         self._children = []     # type: List[SettingDefinition]
         self._relations = []    # type: List[SettingRelation]
@@ -460,7 +460,7 @@ class SettingDefinition:
     #
     #   \return \type{string} The property it depends on or None if it does not depend on another property.
     @classmethod
-    def dependsOnProperty(cls, name: str) -> str:
+    def dependsOnProperty(cls, name: str) -> Optional[str]:
         if name in cls.__property_definitions:
             return cls.__property_definitions[name]["depends_on"]
         return None
@@ -472,7 +472,8 @@ class SettingDefinition:
     #   \param to_string A function that converts a value of this type to a string.
     #
     @classmethod
-    def addSettingType(cls, type_name: str, from_string: Callable[[str], Any], to_string: Callable[[Any],str], validator: Validator = None) -> None:
+    def addSettingType(cls, type_name: str, from_string: Optional[Callable[[str], Any]],
+                       to_string: Callable[[Any], str], validator: Optional[Validator] = None) -> None:
         cls.__type_definitions[type_name] = { "from": from_string, "to": to_string, "validator": validator }
 
     ##  Convert a string to a value according to a setting type.
