@@ -3,8 +3,7 @@
 
 import collections  # For deque, for breadth-first search and to track tasks, and namedtuple.
 import os  # To get the configuration file names and to rename files.
-import shutil
-import tempfile
+import re
 import traceback
 from typing import Any, Dict, Callable, Iterator, List, Optional, Set, Tuple
 
@@ -14,14 +13,12 @@ import UM.i18n  # To translate the "upgrade succeeded" message.
 from UM.Application import Application
 from UM.Logger import Logger
 from UM.MimeTypeDatabase import MimeType
-from UM.Platform import Platform
 from UM.PluginObject import PluginObject
 from UM.PluginRegistry import PluginRegistry  # To find plug-ins.
 from UM.Resources import Resources  # To load old versions from.
 
 catalogue = UM.i18n.i18nCatalog("uranium")
 
-
 ##  File that needs upgrading, with all the required info to upgrade it.
 #
 #   Fields are:
@@ -56,22 +53,20 @@ FilesDataUpdateResult = collections.namedtuple("FilesDataUpdateResult",
 #   to the current (upgraded) versions, where they are never loaded again unless
 #   the user manually retrieves the files.
 class VersionUpgradeManager:
-    ##  The singleton instance of this class.
-    __instance = None   # type: VersionUpgradeManager
-
-    ##  Gets the instance of the VersionUpgradeManager, or creates one.
-    @classmethod
-    def getInstance(cls) -> "VersionUpgradeManager":
-        if not cls.__instance:
-            cls.__instance = VersionUpgradeManager()
-        return cls.__instance
 
     ##  Initialises the version upgrade manager.
     #
     #   This initialises the cache for shortest upgrade routes, and registers
     #   the version upgrade plug-ins.
-    def __init__(self):
-        self._version_upgrades = {} #For each config type and each version, gives a set of upgrade plug-ins that can convert them to something else.
+    def __init__(self, application: Application) -> None:
+        if VersionUpgradeManager.__instance is not None:
+            raise RuntimeError("Try to create singleton '%s' more than once" % self.__class__.__name__)
+        VersionUpgradeManager.__instance = self
+
+        super().__init__()
+
+        self._application = application
+        self._version_upgrades = {} # type: Dict[Tuple[str, int], Set[Tuple[str, int, Callable[[str, List[str]], Optional[Tuple[List[str], List[str]]]]]]]   # For each config type and each version, gives a set of upgrade plug-ins that can convert them to something else.
 
         # For each config type, gives a function with which to get the version number from those files.
         self._get_version_functions = {}  # type: Dict[str, Callable[[str], int]]
@@ -80,14 +75,25 @@ class VersionUpgradeManager:
         self._storage_paths = {}  # type: Dict[str, Dict[int, Set[str]]]
 
         # To know which preference versions and types to upgrade to.
-        self._current_versions = {}
+        self._current_versions = {} # type: Dict[Tuple[str, int], Any]
 
-        self._upgrade_tasks = collections.deque()  # The files that we still have to upgrade.
-        self._upgrade_routes = {}  #type: Dict[Tuple[str, int], Tuple[str, int, Callable[[str, List[str]], Optional[Tuple[List[str], List[str]]]]]] #How to upgrade from one version to another. Needs to be pre-computed after all version upgrade plug-ins are registered.
+        self._upgrade_tasks = collections.deque()  # type: collections.deque  # The files that we still have to upgrade.
+        self._upgrade_routes = {}  # type: Dict[Tuple[str, int], Tuple[str, int, Callable[[str, List[str]], Optional[Tuple[List[str], List[str]]]]]] #How to upgrade from one version to another. Needs to be pre-computed after all version upgrade plug-ins are registered.
 
-        self._registry = PluginRegistry.getInstance()
+        self._registry = PluginRegistry.getInstance()   # type: PluginRegistry
         PluginRegistry.addType("version_upgrade", self._addVersionUpgrade)
 
+        #Regular expressions of the files that should not be checked, such as log files.
+        self._ignored_files = ["^.*\.lock$", "^plugins\.json$", "^packages.json$", "^.*\.log$"]  # type: List[str]
+
+    ##  Registers a file to be ignored by version upgrade checks (eg log files).
+    #   \param file_name The base file name of the file to be ignored.
+
+    def registerIgnoredFile(self, file_name: str) -> None:
+        # Convert the file name to a regular expresion to add to the ignored files
+        file_name_regex = "^" + re.escape(file_name) + "$"
+        self._ignored_files.append(file_name_regex)
+
     ##  Gets the paths where a specified type of file should be stored.
     #
     #   This differs from the storage path in the Resources class, since it also
@@ -107,7 +113,7 @@ class VersionUpgradeManager:
     def setCurrentVersions(self, current_versions) -> None:
         self._current_versions = current_versions
 
-    def registerCurrentVersion(self, version_info: str, type_info: Any) -> None:
+    def registerCurrentVersion(self, version_info: Tuple[str, int], type_info: Any) -> None:
         if version_info in self._current_versions:
             Logger.log("d", "Overwriting current version info: %s", repr(version_info))
         self._current_versions[version_info] = type_info
@@ -181,7 +187,7 @@ class VersionUpgradeManager:
         if "sources" in meta_data:
             for configuration_type, source in meta_data["sources"].items():
                 if "get_version" in source:
-                    self._get_version_functions[configuration_type] = source["get_version"] #May overwrite from other plug-ins that can also load the same configuration type.
+                    self._get_version_functions[configuration_type] = source["get_version"]  # May overwrite from other plug-ins that can also load the same configuration type.
                 if "location" in source:
                     if configuration_type in src_version_dict:
                         src_version = src_version_dict[configuration_type]
@@ -208,14 +214,14 @@ class VersionUpgradeManager:
     #   upgrade said data format one step towards the most recent version, such
     #   that the fewest number of steps is required.
     def _findShortestUpgradeRoutes(self) -> Dict[Tuple[str, int], Tuple[str, int, Callable[[str, List[str]], Optional[Tuple[List[str], List[str]]]]]]:
-        #For each (type, version) tuple, which upgrade function to use to upgrade it towards the newest versions.
-        result = {} #type: Dict[Tuple[str, int], Tuple[str, int, Callable[[str, List[str]], Optional[Tuple[List[str], List[str]]]]]]
+        # For each (type, version) tuple, which upgrade function to use to upgrade it towards the newest versions.
+        result = {}  # type: Dict[Tuple[str, int], Tuple[str, int, Callable[[str, List[str]], Optional[Tuple[List[str], List[str]]]]]]
 
         # Perform a many-to-many shortest path search with Dijkstra's algorithm.
-        front = collections.deque() #type: collections.deque #Use as a queue for breadth-first iteration: Append right, pop left.
+        front = collections.deque()  # type: collections.deque #Use as a queue for breadth-first iteration: Append right, pop left.
         for configuration_type, version in self._current_versions:
             front.append((configuration_type, version))
-        explored_versions = set() #type: Set[Tuple[str, int]]
+        explored_versions = set()  # type: Set[Tuple[str, int]]
         while len(front) > 0:
             destination_type, destination_version = front.popleft()  # To make it a queue, pop on the opposite side of where you append!
             if (destination_type, destination_version) in self._version_upgrades:  # We can upgrade to this version.
@@ -237,9 +243,8 @@ class VersionUpgradeManager:
     #   \param directory The directory to read the files from.
     #   \return The filename of each file relative to the specified directory.
     def _getFilesInDirectory(self, directory: str) -> Iterator[str]:
-        for (path, directory_names, filenames) in os.walk(directory, topdown = True):
-            directory_names[:] = [] # Only go to one level.
-            for filename in filenames:
+        for (path, directory_names, file_names) in os.walk(directory, topdown = True):
+            for filename in file_names:
                 relative_path = os.path.relpath(path, directory)
                 yield os.path.join(relative_path, filename)
 
@@ -256,6 +261,7 @@ class VersionUpgradeManager:
         for key in self._storage_paths:
             self._storage_paths[key] = collections.OrderedDict(sorted(self._storage_paths[key].items()))
 
+        combined_regex_ignored_files = "(" + ")|(".join(self._ignored_files) + ")"
         for old_configuration_type, version_storage_paths_dict in self._storage_paths.items():
             for src_version, storage_paths in version_storage_paths_dict.items():
                 for prefix in storage_path_prefixes:
@@ -264,35 +270,22 @@ class VersionUpgradeManager:
                         for configuration_file in self._getFilesInDirectory(path):
                             # Get file version. Only add this upgrade task if the current file version matches with
                             # the defined version that scans through this folder.
+                            if re.match(combined_regex_ignored_files, configuration_file):
+                                continue
                             try:
                                 with open(os.path.join(path, configuration_file), "r", encoding = "utf-8") as f:
-                                    current_version = self._get_version_functions[old_configuration_type](f.read())
-                                    if current_version != src_version:
-                                        Logger.log("i", "Config file [%s] is of version [%s], which is different from the defined version [%s], no upgrade task for it from type [%s].",
-                                                   configuration_file, current_version, src_version, old_configuration_type)
+                                    file_version = self._get_version_functions[old_configuration_type](f.read())
+                                    if file_version != src_version:
                                         continue
                             except:
                                 Logger.log("w", "Failed to get file version: %s, skip it", configuration_file)
                                 continue
 
+                            Logger.log("i", "Create upgrade task for configuration file [%s] with type [%s] and source version [%s]",
+                                       configuration_file, old_configuration_type, file_version)
                             yield UpgradeTask(storage_path = path, file_name = configuration_file,
                                               configuration_type = old_configuration_type)
 
-    def copyVersionFolder(self, src_path: str, dest_path: str) -> None:
-        Logger.log("i", "Copying directory from '%s' to '%s'", src_path, dest_path)
-        # we first copy everything to a temporary folder, and then move it to the new folder
-        base_dir_name = os.path.basename(src_path)
-        temp_root_dir_path = tempfile.mkdtemp("cura-copy")
-        temp_dir_path = os.path.join(temp_root_dir_path, base_dir_name)
-        # src -> temp -> dest
-        try:
-            shutil.copytree(src_path, temp_dir_path, ignore = shutil.ignore_patterns("*.lock"))
-            # if the dest_path exist, don't overwrite
-            if not os.path.exists(dest_path):
-                shutil.move(temp_dir_path, dest_path)
-        except:
-            Logger.log("e", "Something occurred when copying the version folder from '%s' to '%s'", src_path, dest_path)
-
     ##  Gets the version of the given file data
     def getFileVersion(self, configuration_type: str, file_data: str) -> Optional[int]:
         if configuration_type not in self._get_version_functions:
@@ -386,8 +379,8 @@ class VersionUpgradeManager:
                 # No version upgrade plug-in claims to be able to upgrade this file.
                 return None
             new_type, new_version, upgrade_step = self._upgrade_routes[(configuration_type, version)]
-            new_file_names_without_extension = [] #type: List[str]
-            new_files_data = [] #type: List[str]
+            new_file_names_without_extension = []  # type: List[str]
+            new_files_data = []  # type: List[str]
             for file_idx, file_data in enumerate(files_data):
                 try:
                     upgrade_step_result = upgrade_step(file_data, file_names_without_extension[file_idx])
@@ -422,3 +415,9 @@ class VersionUpgradeManager:
                 return file_name[: -len(suffix) - 1]  # last -1 is for the dot separating name and extension.
 
         return file_name
+
+    __instance = None   # type: VersionUpgradeManager
+
+    @classmethod
+    def getInstance(cls, *args, **kwargs) -> "VersionUpgradeManager":
+        return cls.__instance
