@@ -161,6 +161,10 @@ class RenderBatch():
     #
     #   \param camera The camera to render from.
     def render(self, camera):
+        if camera is None:
+            Logger.log("e", "Unable to render batch without a camera.")
+            return
+
         self._shader.bind()
 
         if self._backface_cull:
@@ -223,6 +227,10 @@ class RenderBatch():
         transformation = item["transformation"]
         mesh = item["mesh"]
 
+        # Do not render if there's no vertex (empty mesh)
+        if mesh.getVertexCount() == 0:
+            return
+
         normal_matrix = None
         if mesh.hasNormals():
             normal_matrix = copy.deepcopy(transformation)
