@@ -1,20 +1,22 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Copyright (c) 2013 David Braam
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from UM.Mesh.MeshReader import MeshReader
-from UM.Mesh.MeshBuilder import MeshBuilder
 import os
-from UM.Scene.SceneNode import SceneNode
 
 from UM.Job import Job
+from UM.Logger import Logger
+from UM.Mesh.MeshReader import MeshReader
+from UM.Mesh.MeshBuilder import MeshBuilder
+from UM.Scene.SceneNode import SceneNode
+
 
 class OBJReader(MeshReader):
-    def __init__(self):
-        super(OBJReader, self).__init__()
+    def __init__(self) -> None:
+        super().__init__()
         self._supported_extensions = [".obj"]
 
-    def read(self, file_name):
+    def _read(self, file_name):
         scene_node = None
 
         extension = os.path.splitext(file_name)[1]
@@ -27,7 +29,7 @@ class OBJReader(MeshReader):
 
             mesh_builder = MeshBuilder()
             mesh_builder.setFileName(file_name)
-            f = open(file_name, "rt")
+            f = open(file_name, "rt", encoding = "utf-8")
             for line in f:
                 parts = line.split()
                 if len(parts) < 1:
@@ -53,7 +55,6 @@ class OBJReader(MeshReader):
 
             mesh_builder.reserveVertexCount(3 * len(face_list))
             num_vertices = len(vertex_list)
-            num_normals = len(normal_list)
 
             for face in face_list:
                 # Substract 1 from index, as obj starts counting at 1 instead of 0
@@ -91,18 +92,24 @@ class OBJReader(MeshReader):
                 else:
                     mesh_builder.addFaceByPoints(vertex_list[i][0], vertex_list[i][1], vertex_list[i][2], vertex_list[j][0], vertex_list[j][1], vertex_list[j][2], vertex_list[k][0], vertex_list[k][1], vertex_list[k][2])
 
-                if ui != -1:
+                if ui != -1 and len(uv_list) > ui:
                     mesh_builder.setVertexUVCoordinates(mesh_builder.getVertexCount() - 3, uv_list[ui][0], uv_list[ui][1])
 
-                if uj != -1:
+                if uj != -1 and len(uv_list) > uj:
                     mesh_builder.setVertexUVCoordinates(mesh_builder.getVertexCount() - 2, uv_list[uj][0], uv_list[uj][1])
 
-                if uk != -1:
+                if uk != -1 and len(uv_list) > uk:
                     mesh_builder.setVertexUVCoordinates(mesh_builder.getVertexCount() - 1, uv_list[uk][0], uv_list[uk][1])
 
                 Job.yieldThread()
             if not mesh_builder.hasNormals():
                 mesh_builder.calculateNormals(fast = True)
+
+            # make sure that the mesh data is not empty
+            if mesh_builder.getVertexCount() == 0:
+                Logger.log("d", "File did not contain valid data, unable to read.")
+                return None  # We didn't load anything.
+
             scene_node.setMeshData(mesh_builder.build())
 
         return scene_node
