@@ -39,7 +39,7 @@ class RotateTool(Tool):
         self._angle = None
         self._angle_update_time = None
 
-        self._shortcut_key = Qt.Key_Z
+        self._shortcut_key = Qt.Key_R
 
         self._progress_message = None
         self._iterations = 0
