@@ -18,9 +18,10 @@ i18n_catalog = i18nCatalog("uranium")
 
 ##  This job checks if there is an update available on the provided URL.
 class UpdateCheckerJob(Job):
-    def __init__(self, silent = False, url = None, callback = None, set_download_url_callback = None):
+    def __init__(self, silent = False, display_same_version = True, url = None, callback = None, set_download_url_callback = None):
         super().__init__()
         self.silent = silent
+        self.display_same_version = display_same_version
         self._url = url
         self._callback = callback
         self._set_download_url_callback = set_download_url_callback
@@ -31,6 +32,7 @@ class UpdateCheckerJob(Job):
         no_new_version = True
 
         application_name = Application.getInstance().getApplicationName()
+
         Logger.log("i", "Checking for new version of %s" % application_name)
         try:
             headers = {"User-Agent": "%s - %s" % (application_name, Application.getInstance().getVersion())}
@@ -67,22 +69,13 @@ class UpdateCheckerJob(Job):
                         if platform.system() == os: #TODO: add architecture check
                             newest_version = Version([int(value["major"]), int(value["minor"]), int(value["revision"])])
                             if local_version < newest_version:
+                                preferences = Application.getInstance().getPreferences()
+                                latest_version_shown = preferences.getValue("info/latest_update_version_shown")
+                                if latest_version_shown == newest_version and not self.display_same_version:
+                                    continue #Don't show this update again. The user already clicked it away and doesn't want it again.
+                                preferences.setValue("info/latest_update_version_shown", str(newest_version))
                                 Logger.log("i", "Found a new version of the software. Spawning message")
-
-                                title_message = i18n_catalog.i18nc("@info:status","Cura {0} is available!", newest_version)
-                                content_message = i18n_catalog.i18nc("@info:status","Cura {0} provides better and reliable printing experience.", newest_version)
-
-                                message = Message(text = content_message, title = title_message)
-                                message.addAction("download", i18n_catalog.i18nc("@action:button", "Download"), "[no_icon]", "[no_description]")
-
-                                message.addAction("new_features", i18n_catalog.i18nc("@action:button", "Learn more about new features"),"[no_icon]", "[no_description]",
-                                                  button_style = Message.ActionButtonStyle.LINK,
-                                                  button_align = Message.ActionButtonStyle.BUTTON_ALIGN_LEFT)
-
-                                if self._set_download_url_callback:
-                                    self._set_download_url_callback(value["url"])
-                                message.actionTriggered.connect(self._callback)
-                                message.show()
+                                self.showUpdate(newest_version, value["url"])
                                 no_new_version = False
                                 break
                     else:
@@ -95,4 +88,21 @@ class UpdateCheckerJob(Job):
             no_new_version = False  # Just to suppress the message below.
 
         if no_new_version and not self.silent:
-            Message(i18n_catalog.i18nc("@info", "No new version was found."), title = i18n_catalog.i18nc("@info:title", "Version Upgrade")).show()
\ No newline at end of file
+            Message(i18n_catalog.i18nc("@info", "No new version was found."), title = i18n_catalog.i18nc("@info:title", "Version Upgrade")).show()
+
+    def showUpdate(self, newest_version: Version, download_url: str) -> None:
+        application_name = Application.getInstance().getApplicationName()
+        title_message = i18n_catalog.i18nc("@info:status","{application_name} {version_number} is available!".format(application_name = application_name.title(), version_number = newest_version))
+        content_message = i18n_catalog.i18nc("@info:status","{application_name} {version_number} provides a better and more reliable printing experience.".format(application_name = application_name.title(), version_number = newest_version))
+
+        message = Message(text = content_message, title = title_message)
+        message.addAction("download", i18n_catalog.i18nc("@action:button", "Download"), "[no_icon]", "[no_description]")
+
+        message.addAction("new_features", i18n_catalog.i18nc("@action:button", "Learn more about the new features"), "[no_icon]", "[no_description]",
+                          button_style = Message.ActionButtonStyle.LINK,
+                          button_align = Message.ActionButtonStyle.BUTTON_ALIGN_LEFT)
+
+        if self._set_download_url_callback:
+            self._set_download_url_callback(download_url)
+        message.actionTriggered.connect(self._callback)
+        message.show()
\ No newline at end of file
